@@ -3,11 +3,14 @@
 
 import logging
 import os
+from typing import Union
 import warnings
 
 from sdsstools import get_config, get_logger, get_package_version
 from sdsstools.configuration import __ENVVARS__
 
+from .exceptions import JaegerUserWarning
+
 
 NAME = "jaeger"
 
@@ -32,6 +35,9 @@ if config["debug"] is False:
     )
 
 
+warnings.simplefilter("always", category=JaegerUserWarning)
+
+
 def start_file_loggers(start_log=True, start_can=True):
 
     if "files" in config and "log_dir" in config["files"]:
@@ -46,6 +52,9 @@ def start_file_loggers(start_log=True, start_can=True):
         can_log.start_file_logger(os.path.join(log_dir, "can.log"))
 
 
+actor_instance = None
+
+
 from .can import *
 from .exceptions import *
 from .fps import *
