@@ -6,15 +6,18 @@
 # @Filename: cli.py
 # @License: BSD 3-clause (http://www.opensource.org/licenses/BSD-3-Clause)
 
+from __future__ import annotations
+
 import asyncio
 import logging
 import os
 import signal
+import socket
 import sys
 import warnings
 from functools import wraps
 
-from typing import Any
+from typing import Any, Optional
 
 import click
 import numpy
@@ -26,7 +29,7 @@ from jaeger import can_log, config, log
 from jaeger.commands.bootloader import load_firmware
 from jaeger.commands.calibration import calibrate_positioner
 from jaeger.exceptions import JaegerError, JaegerUserWarning
-from jaeger.fps import FPS
+from jaeger.fps import FPS, LOCK_FILE
 from jaeger.positioner import Positioner
 from jaeger.testing import VirtualFPS
 
@@ -38,7 +41,7 @@ def shutdown(sign):
     """Shuts down the FPS and stops the positioners in case of a signal interrupt."""
 
     if __FPS__ is not None:
-        __FPS__.send_command("STOP_TRAJECTORY", positioner_ids=0, now=True)
+        __FPS__.send_command("SEND_TRAJECTORY_ABORT", positioner_ids=None, now=True)
         log.error(f"stopping positioners and cancelling due to {sign.name}")
         sys.exit(0)
     else:
@@ -63,14 +66,24 @@ def cli_coro(f):
 class FPSWrapper(object):
     """A helper to store FPS initialisation parameters."""
 
-    def __init__(self, profile, ieb=None, initialise=True, npositioners=10):
+    def __init__(
+        self,
+        profile,
+        ieb=None,
+        initialise=True,
+        npositioners=10,
+        enable_low_temperature=True,
+        use_lock=True,
+    ):
 
         self.profile = profile
         if self.profile in ["test", "virtual"]:
             self.profile = "virtual"
 
         self.ieb = ieb
+        self.enable_low_temperature = enable_low_temperature
         self.initialise = initialise
+        self.use_lock = use_lock
 
         self.vpositioners = []
         self.npositioners = npositioners
@@ -93,7 +106,15 @@ class FPSWrapper(object):
         __FPS__ = self.fps
 
         if self.initialise:
-            await self.fps.initialise()
+            await self.fps.initialise(
+                enable_low_temperature=self.enable_low_temperature,
+                use_lock=self.use_lock,
+            )
+            if self.enable_low_temperature is False:
+                warnings.warn(
+                    "Disabling low temperature handling for sextant.",
+                    JaegerUserWarning,
+                )
 
         return self.fps
 
@@ -159,6 +180,16 @@ pass_fps = click.make_pass_decorator(FPSWrapper, ensure=True)
     help="Use engineering sextant instead of IEB. "
     "Modifies the internal configuration file.",
 )
+@click.option(
+    "--allow-host",
+    is_flag=True,
+    help="Allows running jager in a host other than sdss5-fps.",
+)
+@click.option(
+    "--no-lock",
+    is_flag=True,
+    help="Do not use the lock file, or ignore it if present.",
+)
 @click.pass_context
 def jaeger(
     ctx,
@@ -170,6 +201,8 @@ def jaeger(
     sextant,
     virtual,
     npositioners,
+    allow_host,
+    no_lock,
 ):
     """CLI for the SDSS-V focal plane system.
 
@@ -177,6 +210,12 @@ def jaeger(
 
     """
 
+    if allow_host is False:
+        hostname = socket.getfqdn()
+        if hostname.endswith("apo.nmsu.edu") or hostname.endswith("lco.cl"):
+            if not hostname.startswith("sdss5-fps"):
+                raise RuntimeError("At the observatories jaeger must run on sdss5-fps.")
+
     if verbose > 0 and quiet:
         raise click.UsageError("--quiet and --verbose are mutually exclusive.")
 
@@ -206,6 +245,11 @@ def jaeger(
         config["files"]["ieb_config"] = sextant_file
         log.debug(f"Using internal IEB sextant onfiguration file {sextant_file}.")
 
+    if sextant or "sextants/" in config["files"]["ieb_config"]:
+        enable_low_temperature = False
+    else:
+        enable_low_temperature = True
+
     if virtual is True:
         profile = "virtual"
 
@@ -213,6 +257,8 @@ def jaeger(
         profile,
         ieb=ieb,
         npositioners=npositioners,
+        enable_low_temperature=enable_low_temperature,
+        use_lock=not no_lock,
     )
 
 
@@ -628,5 +674,106 @@ async def status(fps_maker: FPSWrapper, positioner_id: int):
             raise JaegerError(f"Failed retrieving position: {err}")
 
 
+@jaeger.command()
+@click.option(
+    "--collision-buffer",
+    type=click.FloatRange(1.6, 3.0),
+    help="Custom collision buffer",
+)
+@click.option(
+    "--force",
+    is_flag=True,
+    help="Execute unwind even in presence of deadlocks.",
+)
+@pass_fps
+@cli_coro
+async def unwind(fps_maker, collision_buffer: float | None = None, force: bool = False):
+    """Unwinds the array."""
+
+    from jaeger.kaiju import unwind
+
+    async with fps_maker as fps:
+
+        await fps.update_position()
+        positions = {p.positioner_id: (p.alpha, p.beta) for p in fps.values()}
+
+        log.info("Calculating trajectory.")
+        trajectory = await unwind(
+            positions,
+            collision_buffer=collision_buffer,
+            force=force,
+        )
+
+        log.info("Executing unwind trajectory.")
+        await fps.send_trajectory(trajectory)
+
+    return
+
+
+@jaeger.command()
+@click.argument("EXPLODE-DEG", type=float)
+@pass_fps
+@cli_coro
+async def explode(fps_maker, explode_deg: float):
+    """Explodes the array."""
+
+    from jaeger.kaiju import explode
+
+    async with fps_maker as fps:
+
+        await fps.update_position()
+        positions = {p.positioner_id: (p.alpha, p.beta) for p in fps.values()}
+
+        log.info("Calculating trajectory.")
+        trajectory = await explode(positions, explode_deg=explode_deg)
+
+        log.info("Executing explode trajectory.")
+        await fps.send_trajectory(trajectory)
+
+    return
+
+
+@jaeger.command()
+@click.argument("PATH", required=False, type=click.Path(exists=False, dir_okay=False))
+@click.option("--collision-buffer", type=float, help="The collision buffer.")
+@pass_fps
+@cli_coro
+async def snapshot(
+    fps_maker: FPSWrapper,
+    path: Optional[str] = None,
+    collision_buffer: float | None = None,
+):
+    """Takes a snapshot image."""
+
+    if path is not None:
+        path = str(path)
+
+    async with fps_maker as fps:
+
+        await fps.update_position()
+        filename = await fps.save_snapshot(path, collision_buffer=collision_buffer)
+
+    print(f"Snapshot saved to {filename}")
+
+
+@jaeger.command()
+@pass_fps
+@cli_coro
+async def unlock(fps_maker: FPSWrapper):
+    """Unlocks the FPS."""
+
+    warnings.filterwarnings(
+        "ignore",
+        message=".+FPS was collided and has been locked.+",
+        category=JaegerUserWarning,
+    )
+
+    async with fps_maker as fps:
+        await fps.unlock()
+
+    if os.path.exists(LOCK_FILE):
+        os.remove(LOCK_FILE)
+
+
 if __name__ == "__main__":
     jaeger()
