@@ -12,9 +12,13 @@ from clu.parsers.click import command_parser
 jaeger_parser = command_parser
 
 
+from .can import *
+from .configuration import *
 from .debug import *
 from .fvc import *
 from .ieb import *
 from .pollers import *
 from .positioner import *
+from .snapshot import *
 from .talk import *
+from .unwind import *
