@@ -70,6 +70,18 @@ def ieb(command, fps):
     return
 
 
+@ieb.command()
+async def enable(command, fps):
+    """Re-enables the IEB."""
+
+    if fps.ieb is None or not isinstance(fps.ieb, IEB):
+        return command.fail("IEB object does not exist.")
+
+    fps.ieb.disabled = False
+
+    return command.finish()
+
+
 @ieb.command()
 async def status(command, fps):
     """Outputs the status of the devices."""
@@ -268,7 +280,8 @@ def power(command, fps):
 
 
 @power.command()
-async def on(command, fps: FPS):
+@click.option("--no-gfas", is_flag=True, help="Do not power the GFAs.")
+async def on(command, fps: FPS, no_gfas: bool = False):
     """Powers on all the FPS IEB components."""
 
     command.info(text="Turning pollers off.")
@@ -288,12 +301,6 @@ async def on(command, fps: FPS):
         "PS4",
         "PS5",
         "PS6",
-        "GFA1",
-        "GFA2",
-        "GFA3",
-        "GFA4",
-        "GFA5",
-        "GFA6",
         "NUC1",
         "NUC2",
         "NUC3",
@@ -302,6 +309,16 @@ async def on(command, fps: FPS):
         "NUC6",
     ]
 
+    if no_gfas is False:
+        on_seq += [
+            "GFA1",
+            "GFA2",
+            "GFA3",
+            "GFA4",
+            "GFA5",
+            "GFA6",
+        ]
+
     if not (await _power_sequence(command, ieb, on_seq, mode="on")):
         return
 
