@@ -176,7 +176,7 @@ async def goto(
     except JaegerError as err:
         return command.fail(error=f"Goto command failed: {err}")
 
-    command.finish(text="Position reached")
+    command.finish(text="Destination reached")
 
 
 @jaeger_parser.command()
@@ -258,8 +258,11 @@ async def status(command, fps, positioner_ids, full):
     if not check_positioners(positioner_ids, command, fps):
         return
 
-    await fps.update_status(positioner_ids=positioner_ids)
-    await fps.update_position(positioner_ids=positioner_ids)
+    try:
+        await fps.update_status(positioner_ids=0)
+        await fps.update_position(positioner_ids=positioner_ids)
+    except JaegerError as err:
+        return command.fail(error=f"Failed reporting status: {err}")
 
     n_trajs = (
         await fps.send_command(
@@ -289,11 +292,11 @@ async def status(command, fps, positioner_ids, full):
 
         command.write(
             "i",
-            status=[
+            positioner_status=[
                 p.positioner_id,
                 alpha_pos,
                 beta_pos,
-                int(p.status),
+                f"0x{int(p.status):x}",
                 p.initialised,
                 p.is_bootloader() or False,
                 p.firmware or "?",
@@ -303,7 +306,7 @@ async def status(command, fps, positioner_ids, full):
             ],
         )
 
-    command.info(low_temperature=command.actor.low_temperature.value)
+    command.info(fps_status=f"0x{fps.status.value:x}")
 
     if full:
         await clu.Command("ieb status", parent=command).parse()
@@ -361,10 +364,6 @@ async def stop(command, fps):
 async def unlock(command, fps):
     """Unlocks the FPS."""
 
-    if not fps.locked:
-        command.info(locked=False)
-        return command.finish(text="FPS is not locked")
-
     result = await fps.unlock()
 
     if result:
