@@ -165,6 +165,27 @@ class JaegerCAN(Generic[Bus_co]):
 
         return self
 
+    def stop(self):
+        """Stops the interfaces."""
+
+        if self.notifier:
+            self.notifier.stop()
+            self.notifier = None
+
+        for interface in self.interfaces:
+            interface: Any
+            try:
+                interface.close()
+            except AttributeError:
+                pass
+
+        self.interfaces = []
+
+        if self._command_queue_task:
+            self._command_queue_task.cancel()
+
+        self._started = False
+
     @classmethod
     async def create(
         cls,
@@ -299,7 +320,7 @@ class JaegerCAN(Generic[Bus_co]):
 
             log.error(
                 f"A collision was detected in positioner {positioner_id}. "
-                "Sending STOP_TRAJECTORIES and locking the FPS."
+                "Sending SEND_TRAJECTORY_ABORT and locking the FPS."
             )
 
             if self.fps:
@@ -505,6 +526,12 @@ class CANnetInterface(JaegerCAN[CANNetBus]):
         )
         self.device_status_poller.start()
 
+    def stop(self):
+        """Stops the interfaces."""
+
+        super().stop()
+        asyncio.create_task(self.device_status_poller.stop())
+
     async def _process_reply_queue(self, msg: Message):
         """Processes a message checking first if it comes from the device."""
 
