@@ -234,7 +234,7 @@ async def goto(
     beta: float | list | numpy.ndarray,
     speed: Optional[Tuple[float, float]] = None,
     relative: bool = False,
-    use_sync_line: bool = True,
+    use_sync_line: bool = None,
 ):
     """Send positioners to a given position using a trajectory.
 
@@ -258,9 +258,6 @@ async def goto(
 
     """
 
-    # Just in case.
-    await fps.start_can()
-
     if not isinstance(alpha, (list, tuple, numpy.ndarray)):
         alpha = numpy.tile(alpha, len(positioner_ids))
     if not isinstance(beta, (list, tuple, numpy.ndarray)):
