@@ -11,6 +11,7 @@ from __future__ import annotations
 import asyncio
 import pathlib
 import time
+import warnings
 
 from typing import TYPE_CHECKING, Dict, List, Tuple, cast
 
@@ -21,9 +22,9 @@ from sdsstools import read_yaml_file
 
 from jaeger import config, log
 from jaeger.commands import Command, CommandID
-from jaeger.exceptions import FPSLockedError, TrajectoryError
+from jaeger.exceptions import FPSLockedError, JaegerUserWarning, TrajectoryError
 from jaeger.ieb import IEB
-from jaeger.maskbits import PositionerStatus, ResponseCode
+from jaeger.maskbits import FPSStatus, PositionerStatus, ResponseCode
 from jaeger.utils import int_to_bytes
 
 
@@ -36,7 +37,7 @@ __all__ = [
     "SendNewTrajectory",
     "SendTrajectoryData",
     "TrajectoryDataEnd",
-    "TrajectoryTransmissionAbort",
+    "SendTrajectoryAbort",
     "StartTrajectory",
     "StopTrajectory",
     "Trajectory",
@@ -53,7 +54,7 @@ TrajectoryDataType = Dict[int, Dict[str, List[Tuple[float, float]]]]
 async def send_trajectory(
     fps: FPS,
     trajectories: str | pathlib.Path | TrajectoryDataType,
-    use_sync_line=True,
+    use_sync_line: bool | None = None,
     send_trajectory=True,
     start_trajectory=True,
 ) -> Trajectory:
@@ -78,7 +79,7 @@ async def send_trajectory(
     use_sync_line
         If `True`, the SYNC line will be used to synchronise the beginning of
         all trajectories. Otherwise a `.START_TRAJECTORY` command will be sent
-        over the CAN network.
+        over the CAN network. If `None`, defaults to the configuration parameter.
     send_trajectory
         If `True`, sends the trajectory to the positioners and returns the
         `.Trajectory` instance.
@@ -109,6 +110,9 @@ async def send_trajectory(
 
     traj = Trajectory(fps, trajectories)
 
+    if use_sync_line is None:
+        use_sync_line = config["fps"]["use_sync_line"]
+
     if use_sync_line:
         if not isinstance(fps.ieb, IEB) or fps.ieb.disabled:
             raise TrajectoryError("IEB is not connected. Cannot use SYNC line.", traj)
@@ -128,7 +132,7 @@ async def send_trajectory(
             err.trajectory,
         )
 
-    log.debug(f"Trajectory successfully sent in {traj.data_send_time:1f} seconds.")
+    log.debug(f"Trajectory sent in {traj.data_send_time:1f} seconds.")
     log.info(f"Expected time to complete trajectory: {traj.move_time:.2f} seconds.")
 
     if start_trajectory is False:
@@ -150,7 +154,7 @@ async def send_trajectory(
             err.trajectory,
         )
 
-    log.info("All positioners have successfully reached their positions.")
+    log.info("All positioners have reached their destinations.")
 
     return traj
 
@@ -302,7 +306,9 @@ class Trajectory(object):
             timeout=1.0,
         ):
             self.failed = True
-            raise TrajectoryError("some positioners did not respond.", self)
+            raise TrajectoryError("Some positioners did not respond.", self)
+
+        log.debug("Sending data points:")
 
         # Check that all positioners are ready to receive a new trajectory.
         for pos_id in self.trajectories:
@@ -313,7 +319,7 @@ class Trajectory(object):
             if positioner.disabled:
                 self.failed = True
                 raise TrajectoryError(
-                    f"positioner_id={pos_id} is disabled but "
+                    f"positioner_id={pos_id} is disabled but was "
                     "included in the trajectory.",
                     self,
                 )
@@ -332,6 +338,8 @@ class Trajectory(object):
 
             traj_pos = self.trajectories[pos_id]
 
+            log.debug(f"{pos_id}: {traj_pos}")
+
             self.n_points[pos_id] = (len(traj_pos["alpha"]), len(traj_pos["beta"]))
 
             # Gets maximum time for this positioner
@@ -446,8 +454,8 @@ class Trajectory(object):
         if not self._ready_to_start or self.failed:
             raise TrajectoryError("The trajectory has not been sent.", self)
 
-        for positioner_id in list(self.trajectories.keys()):
-            self.fps[positioner_id].move_time = self.move_time
+        if self.move_time is None:
+            raise TrajectoryError("move_time not set.", self)
 
         if use_sync_line:
 
@@ -465,10 +473,8 @@ class Trajectory(object):
             # Set SYNC line to high.
             await sync.close()
 
-            await asyncio.sleep(0.5)
-
-            # Reset SYNC line.
-            await sync.open()
+            # Schedule reseting of SYNC line
+            asyncio.get_event_loop().call_later(0.5, asyncio.create_task, sync.open())
 
         else:
 
@@ -491,9 +497,51 @@ class Trajectory(object):
 
         self.start_time = time.time()
 
+        min_trajectory_time = 2.0
+        PS = PositionerStatus
+
         try:
-            if self.move_time is None:
-                raise TrajectoryError("move_time not set.", self)
+
+            # The positioners take a bit to report that they are moving so if the
+            # move time is too short, we don't try to check if the positioners started
+            # moving (but we'll check later that they arrived to their positions.)
+            if self.move_time >= min_trajectory_time:
+                await asyncio.sleep(min_trajectory_time)
+                await self.fps.update_status()
+
+                if self.fps.status & FPSStatus.IDLE:
+                    raise TrajectoryError("Move failed to start.")
+
+                sbits = numpy.array([p.status for p in self.fps.values()])
+                not_moving = numpy.where(sbits & PS.DISPLACEMENT_COMPLETED)
+                if not_moving[0].any():
+                    not_moving_pids = numpy.array(list(self.fps))[not_moving[0]]
+
+                    # Before reporting that the positioner is not moving, check if it's
+                    # already there.
+                    await self.fps.update_position()
+                    positions = self.fps.get_positions()
+
+                    really_not_moving = []
+                    for pid in not_moving_pids:
+                        if pid not in self.trajectories:
+                            continue
+
+                        current = positions[positions[:, 0] == pid][0, 1:]
+                        alpha = self.trajectories[pid]["alpha"][-1][0]
+                        beta = self.trajectories[pid]["beta"][-1][0]
+
+                        if not numpy.allclose(current - [alpha, beta], 0, atol=0.1):
+                            really_not_moving.append(pid)
+
+                    if len(really_not_moving) > 0:
+                        not_moving_str = ", ".join(map(str, really_not_moving))
+                        # Should this be an error?
+                        warnings.warn(
+                            "Some positioners do not appear to be "
+                            f"moving: {not_moving_str}.",
+                            JaegerUserWarning,
+                        )
 
             while True:
 
@@ -507,27 +555,53 @@ class Trajectory(object):
 
                 await self.fps.update_status()
 
-                if all(
-                    [
-                        PositionerStatus.DISPLACEMENT_COMPLETED in self.fps[pid].status
-                        for pid in self.trajectories
-                    ]
-                ):
+                if self.fps.status & FPSStatus.IDLE:
                     self.failed = False
                     break
 
-            elapsed = time.time() - self.start_time
-            if elapsed > (self.move_time + 3):
+                elapsed = time.time() - self.start_time
+                if elapsed > (self.move_time + 3):
+                    raise TrajectoryError(
+                        "Some positioners did not complete the move.",
+                        self,
+                    )
+
+            # TODO: There seems to be bug in the firmware. Sometimes when a positioner
+            # fails to start its trajectory, at the end of the trajectory time it
+            # does believe it has reached the commanded position, although it's still
+            # at the initial position. In those cases issuing a SEND_TRAJECTORY_ABORT
+            # followed by a position update seems to return correct positions.
+            await self.fps.stop_trajectory()
+
+            # The FPS says they have all stopped moving but check that they are
+            # actually at their positions.
+            await self.fps.update_position()
+            failed_reach = False
+            for pid in self.trajectories:
+                alpha = self.trajectories[pid]["alpha"][-1][0]
+                beta = self.trajectories[pid]["beta"][-1][0]
+                current_position = numpy.array(self.fps[pid].position)
+                if not numpy.allclose(current_position, [alpha, beta], atol=0.1):
+                    warnings.warn(
+                        f"Positioner {pid} may not have reached its destination.",
+                        JaegerUserWarning,
+                    )
+                    failed_reach = True
+
+            if failed_reach:
                 raise TrajectoryError(
-                    "Some positioners did not complete the move.",
+                    "Some positioners did not reach their destinations.",
                     self,
                 )
 
         except BaseException:
             self.failed = True
+            await self.fps.stop_trajectory()
             raise
 
         finally:
+            await self.fps.stop_trajectory()
+            await self.fps.update_position()
             self.end_time = time.time()
             if restart_pollers:
                 self.fps.pollers.start()
@@ -538,7 +612,7 @@ class Trajectory(object):
         """Aborts the trajectory transmission."""
 
         cmd = await self.fps.send_command(
-            "TRAJECTORY_TRANSMISSION_ABORT",
+            "SEND_TRAJECTORY_ABORT",
             positioner_ids=list(self.trajectories.keys()),
         )
 
@@ -632,10 +706,10 @@ class TrajectoryDataEnd(Command):
     move_command = True
 
 
-class TrajectoryTransmissionAbort(Command):
+class SendTrajectoryAbort(Command):
     """Aborts sending a trajectory."""
 
-    command_id = CommandID.TRAJECTORY_TRANSMISSION_ABORT
+    command_id = CommandID.SEND_TRAJECTORY_ABORT
     broadcastable = False
     move_command = False
     safe = True
@@ -647,6 +721,7 @@ class StartTrajectory(Command):
     command_id = CommandID.START_TRAJECTORY
     broadcastable = True
     move_command = True
+    safe = True
 
 
 class StopTrajectory(Command):
