@@ -13,11 +13,27 @@ import os
 import pathlib
 import warnings
 from dataclasses import dataclass
+from glob import glob
 
-from typing import Any, ClassVar, Dict, List, Optional, Tuple, Type, TypeVar, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    ClassVar,
+    Dict,
+    Generic,
+    List,
+    Optional,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 
 import numpy
+from astropy.time import Time
+from zc.lockfile import LockFile
 
+import jaeger
 from jaeger import can_log, config, log, start_file_loggers
 from jaeger.can import JaegerCAN
 from jaeger.commands import (
@@ -29,30 +45,44 @@ from jaeger.commands import (
 )
 from jaeger.exceptions import (
     FPSLockedError,
-    JaegerDeprecationWarning,
     JaegerError,
     JaegerUserWarning,
     PositionerError,
 )
 from jaeger.ieb import IEB
 from jaeger.interfaces import BusABC
+from jaeger.maskbits import FPSStatus, PositionerStatus
 from jaeger.positioner import Positioner
 from jaeger.utils import Poller, PollerList
 
 
+if TYPE_CHECKING:
+    from matplotlib.axes import Axes
+
+    from jaeger.target.configuration import BaseConfiguration
+
+
 __all__ = ["BaseFPS", "FPS"]
 
 
 MIN_BETA = 160
+LOCK_FILE = "/var/tmp/sdss/jaeger.lock"
+
+FPS_CO = Union["BaseFPS", "FPS"]
+FPS_T = TypeVar("FPS_T", bound="BaseFPS")
 
 
-class BaseFPS(dict):
+class BaseFPS(Dict[int, Positioner], Generic[FPS_T]):
     """A class describing the Focal Plane System.
 
     This class includes methods to read the layout and construct positioner
     objects and can be used by the real `FPS` class or the
     `~jaeger.testing.VirtualFPS`.
 
+    `.BaseFPS` instances are singletons in the sense that one cannot instantiate
+    more than one. An error is raise if ``__new__`` is called with an existing
+    instance. To retrieve the running instance, use `.get_instance`.
+
     Attributes
     ----------
     positioner_class
@@ -63,9 +93,41 @@ class BaseFPS(dict):
     """
 
     positioner_class: ClassVar[Type[Positioner]] = Positioner
+    _instance: ClassVar[dict[Type[BaseFPS], FPS_T]] = {}
 
-    def __init__(self):
-        dict.__init__(self, {})
+    initialised: bool
+
+    def __new__(cls: Type[FPS_T], *args, **kwargs):
+
+        if cls in cls._instance and kwargs == {}:
+            raise JaegerError(
+                "An instance of FPS is already running. "
+                "Use get_instance() to retrieve it."
+            )
+
+        if cls not in cls._instance or kwargs != {}:
+
+            if cls not in cls._instance:
+                new_obj = super().__new__(cls)
+                cls._instance[cls] = new_obj
+            else:
+                new_obj = cls._instance[cls]
+
+            dict.__init__(new_obj, {})
+            new_obj.initialised = False
+
+            return new_obj
+
+        raise RuntimeError("Returning None in BaseFPS.__new__. This should not happen.")
+
+    @classmethod
+    def get_instance(cls: Type[FPS_T], *args, **kwargs) -> FPS_T:
+        """Returns the running instance."""
+
+        if cls not in cls._instance or kwargs:
+            return cls(*args, **kwargs)
+
+        return cls._instance[cls]
 
     @property
     def positioners(self):
@@ -107,7 +169,7 @@ T = TypeVar("T", bound="FPS")
 
 
 @dataclass
-class FPS(BaseFPS):
+class FPS(BaseFPS["FPS"]):
     """A class describing the Focal Plane System.
 
     The recommended way to instantiate a new `.FPS` object is to use the `.create`
@@ -147,8 +209,12 @@ class FPS(BaseFPS):
 
     """
 
+    # __instance: ClassVar[dict[Type[FPS], FPS]] = {}
+
     can: JaegerCAN | str | None = None
     ieb: Union[bool, IEB, dict, str, pathlib.Path, None] = None
+    configuration: BaseConfiguration | None = None
+    status: FPSStatus = FPSStatus.IDLE | FPSStatus.TEMPERATURE_NORMAL
 
     def __post_init__(self):
 
@@ -168,6 +234,8 @@ class FPS(BaseFPS):
         # The mapping between positioners and buses.
         self.positioner_to_bus: Dict[int, Tuple[BusABC, int | None]] = {}
 
+        self.pid_lock: LockFile | None = None
+
         self._locked = False
         self.locked_by: List[int] = []
 
@@ -197,6 +265,9 @@ class FPS(BaseFPS):
 
         assert isinstance(self.ieb, IEB) or self.ieb is None
 
+        self.__status_event = asyncio.Event()
+        self.__temperature_task: asyncio.Task | None = None
+
         # Position and status pollers
         self.pollers = PollerList(
             [
@@ -222,6 +293,8 @@ class FPS(BaseFPS):
         ieb=None,
         initialise=True,
         start_pollers: bool | None = None,
+        enable_low_temperature: bool = True,
+        use_lock: bool = True,
     ) -> "FPS":
         """Starts the CAN bus and .
 
@@ -231,22 +304,39 @@ class FPS(BaseFPS):
             Whether to initialise the FPS.
         start_pollers
             Whether to initialise the pollers.
+        use_lock
+            Use a lock file to prevent multiple instances of `.FPS`.
         kwargs
             Parameters to pass to `.FPS`.
 
         """
 
         instance = cls(can=can, ieb=ieb)
-        await instance.start_can()
+        await instance.start_can(use_lock=use_lock)
 
         if initialise:
-            await instance.initialise(start_pollers=start_pollers)
+            await instance.initialise(
+                start_pollers=start_pollers,
+                enable_low_temperature=enable_low_temperature,
+            )
 
         return instance
 
-    async def start_can(self):
+    async def start_can(self, use_lock: bool = True):
         """Starts the JaegerCAN interface."""
 
+        if use_lock and self.pid_lock is None:
+            try:
+                if not os.path.exists(os.path.dirname(LOCK_FILE)):
+                    os.makedirs(os.path.dirname(LOCK_FILE))
+                self.pid_lock = LockFile(LOCK_FILE)
+            except Exception:
+                raise JaegerError(
+                    f"Failed creating lock file {LOCK_FILE}. "
+                    "Probably another instance is running. "
+                    "If that is not the case, remove the lock file and retry."
+                )
+
         if isinstance(self.can, JaegerCAN):
             if self.can._started:
                 return
@@ -255,6 +345,7 @@ class FPS(BaseFPS):
                 return
 
         self.can = await JaegerCAN.create(self.can, fps=self)
+        return True
 
     def add_positioner(
         self,
@@ -277,7 +368,12 @@ class FPS(BaseFPS):
 
         return positioner
 
-    async def initialise(self: T, start_pollers: bool | None = None) -> T:
+    async def initialise(
+        self: T,
+        start_pollers: bool | None = None,
+        enable_low_temperature: bool = True,
+        use_lock: bool = True,
+    ) -> T:
         """Initialises all positioners with status and firmware version.
 
         Parameters
@@ -300,7 +396,7 @@ class FPS(BaseFPS):
             await self.pollers.stop()
 
         # Make sure CAN buses are connected.
-        await self.start_can()
+        await self.start_can(use_lock=use_lock)
 
         # Test IEB connection.
         if isinstance(self.ieb, IEB):
@@ -356,6 +452,9 @@ class FPS(BaseFPS):
             positioner.fps = self
             positioner.firmware = get_fw_command.get_firmware()[reply.positioner_id]
 
+            if positioner.positioner_id in config["fps"]["disabled_positioners"]:
+                positioner.disabled = True
+
         if len(ignored_positioners) > 0:
             warnings.warn(
                 "The following connected positioners are ignored as they are "
@@ -363,6 +462,9 @@ class FPS(BaseFPS):
                 JaegerUserWarning,
             )
 
+        # Mark as initialised here although we have some more work to do.
+        self.initialised = True
+
         pids = sorted(list(self.keys()))
         if len(pids) > 0:
             log.info(f"Found {len(pids)} connected positioners: {pids!r}.")
@@ -376,15 +478,15 @@ class FPS(BaseFPS):
                 JaegerUserWarning,
             )
 
-        # Stop all positioners just in case.
+        # Stop all positioners just in case. This won't clear collided flags.
         if not self.is_bootloader():
             await self.stop_trajectory()
 
         # Initialise positioners
         try:
             disable_precise_moves = config["positioner"]["disable_precise_moves"]
-            if disable_precise_moves:
-                warnings.warn("Disabling precise moves.", JaegerUserWarning)
+            # if disable_precise_moves:
+            #     warnings.warn("Disabling precise moves.", JaegerUserWarning)
             pos_initialise = [
                 positioner.initialise(disable_precise_moves=disable_precise_moves)
                 for positioner in self.values()
@@ -418,6 +520,19 @@ class FPS(BaseFPS):
             )
             return self
 
+        # Check if any of the positioners are collided and if so lock the FPS.
+        locked_by = []
+        for positioner in self.values():
+            if positioner.collision:
+                locked_by.append(positioner.positioner_id)
+
+        if len(locked_by) > 0:
+            await self.lock(by=locked_by, do_warn=False, snapshot=False)
+            warnings.warn(
+                "The FPS was collided and has been locked.",
+                JaegerUserWarning,
+            )
+
         if config.get("safe_mode", False) is not False:
             min_beta = MIN_BETA
             if isinstance(config["safe_mode"], dict):
@@ -444,12 +559,47 @@ class FPS(BaseFPS):
                 positioner_ids=disable_connected,
             )
 
+        # Start temperature watcher.
+        if self.__temperature_task is not None:
+            self.__temperature_task.cancel()
+        if (
+            isinstance(self.ieb, IEB)
+            and not self.ieb.disabled
+            and enable_low_temperature
+        ):
+            self.__temperature_task = asyncio.create_task(self._handle_temperature())
+        else:
+            self.set_status(
+                (self.status & ~FPSStatus.TEMPERATURE_NORMAL)
+                | FPSStatus.TEMPERATURE_UNKNOWN
+            )
+
+        # Issue an update status to get the status set.
+        await self.update_status()
+
         # Start the pollers
         if start_pollers and not self.is_bootloader():
             self.pollers.start()
 
         return self
 
+    def set_status(self, status: FPSStatus):
+        """Sets the status of the FPS."""
+
+        if status != self.status:
+            self.status = status
+            if not self.__status_event.is_set():
+                self.__status_event.set()
+
+    async def async_status(self):
+        """Generator that yields FPS status changes."""
+
+        yield self.status
+        while True:
+            await self.__status_event.wait()
+            yield self.status
+            self.__status_event.clear()
+
     async def _get_positioner_bus_map(self):
         """Creates the positioner-to-bus map.
 
@@ -482,9 +632,7 @@ class FPS(BaseFPS):
     def moving(self):
         """Returns `True` if any of the positioners is moving."""
 
-        return any(
-            [pos.moving for pos in self.values() if pos.status != pos.flags.UNKNOWN]
-        )
+        return self.status & FPSStatus.MOVING
 
     def is_bootloader(self):
         """Returns `True` if any positioner is in bootloader mode."""
@@ -495,7 +643,6 @@ class FPS(BaseFPS):
         self,
         command: str | int | CommandID | Command,
         positioner_ids: int | List[int] | None = None,
-        positioner_id: int | List[int] | None = None,
         data: Any = None,
         now: bool = False,
         **kwargs,
@@ -535,16 +682,8 @@ class FPS(BaseFPS):
 
         """
 
-        assert isinstance(self.can, JaegerCAN), "CAN connection not established."
-
-        if positioner_id is not None:
-            warnings.warn(
-                "positioner_id is deprecated and will be removed soon. "
-                "Use positioner_ids.",
-                JaegerDeprecationWarning,
-            )
-            if positioner_ids is None:
-                positioner_ids = positioner_id
+        if not isinstance(self.can, JaegerCAN) or self.can._started is False:
+            raise JaegerError("CAN connection not established.")
 
         if positioner_ids is None:
             positioner_ids = [p for p in self if not self[p].disabled]
@@ -616,17 +755,22 @@ class FPS(BaseFPS):
         self,
         stop_trajectories: bool = True,
         by: Optional[List[int]] = None,
+        do_warn: bool = True,
+        snapshot: bool = True,
     ):
         """Locks the `.FPS` and prevents commands to be sent.
 
         Parameters
         ----------
         stop_trajectories
-            Whether to stop trajectories when locking.
+            Whether to stop trajectories when locking. This will not
+            clear any collided flags.
 
         """
 
-        warnings.warn("Locking the FPS.", JaegerUserWarning)
+        self._locked = True
+        if do_warn:
+            warnings.warn("Locking the FPS.", JaegerUserWarning)
 
         if stop_trajectories:
             await self.stop_trajectory()
@@ -634,18 +778,36 @@ class FPS(BaseFPS):
         if by:
             self.locked_by += by
 
-        self._locked = True
+        await self.update_status()
+
+        if jaeger.actor_instance:
+            jaeger.actor_instance.write("e", {"locked": True})
+
+        if snapshot:
+            if by is not None and len(by) > 0:
+                highlight = by[0]
+            else:
+                highlight = None
+            filename = await self.save_snapshot(highlight=highlight)
+            warnings.warn(f"Snapshot for locked FPS: {filename}", JaegerUserWarning)
+
+            if jaeger.actor_instance:
+                jaeger.actor_instance.write("i", {"snapshot": filename})
 
     async def unlock(self, force=False):
         """Unlocks the `.FPS` if all collisions have been resolved."""
 
-        await self.update_status(timeout=0.1)
+        # Send STOP_TRAJECTORY. This clears the collided flags.
+        await self.stop_trajectory(clear_flags=True)
+
+        await self.update_status(timeout=2)
 
         for positioner in self.positioners.values():
             if positioner.collision:
                 self._locked = True
                 raise JaegerError(
-                    "Cannot unlock the FPS until all the collisions have been cleared."
+                    "Cannot unlock the FPS until all the "
+                    "collisions have been cleared."
                 )
 
         self._locked = False
@@ -653,6 +815,17 @@ class FPS(BaseFPS):
 
         return True
 
+    def get_positions(self, ignore_disabled=False) -> numpy.ndarray:
+        """Returns the alpha and beta positions as an array."""
+
+        data = [
+            (p.positioner_id, p.alpha, p.beta)
+            for p in self.positioners.values()
+            if ignore_disabled is False or p.disabled is False
+        ]
+
+        return numpy.array(data)
+
     async def update_status(
         self,
         positioner_ids: Optional[int | List[int]] = None,
@@ -670,6 +843,9 @@ class FPS(BaseFPS):
 
         """
 
+        if len(self.positioners) == 0:
+            return True
+
         if positioner_ids is None:
             positioner_ids = [0]
         elif not isinstance(positioner_ids, (list, tuple)):
@@ -694,6 +870,9 @@ class FPS(BaseFPS):
             log.warning(f"{CommandID.GET_STATUS.name!r} failed during update status.")
             return False
 
+        if len(command.replies) == 0:
+            return True
+
         update_status_coros = []
         for pid, status_int in command.get_positioner_status().items():  # type: ignore
 
@@ -704,13 +883,30 @@ class FPS(BaseFPS):
 
         await asyncio.gather(*update_status_coros)
 
+        # Set the status of the FPS based on positioner information.
+        # First get the current bitmask without the status bit.
+        current = self.status & ~FPSStatus.STATUS_BITS
+
+        pbits = numpy.array([int(p.status) for p in self.values()])
+
+        coll_bits = PositionerStatus.COLLISION_ALPHA | PositionerStatus.COLLISION_BETA
+
+        if ((pbits & coll_bits) > 0).any():
+            self.set_status(current | FPSStatus.COLLIDED)
+
+        elif ((pbits & PositionerStatus.DISPLACEMENT_COMPLETED) > 0).all():
+            self.set_status(current | FPSStatus.IDLE)
+
+        else:
+            self.set_status(current | FPSStatus.MOVING)
+
         return True
 
     async def update_position(
         self,
         positioner_ids: Optional[int | List[int]] = None,
         timeout: float = 1,
-    ) -> bool:
+    ) -> numpy.ndarray | bool:
         """Updates positions.
 
         Parameters
@@ -723,6 +919,9 @@ class FPS(BaseFPS):
 
         """
 
+        if len(self.positioners) == 0:
+            return True
+
         if positioner_ids is None:
             positioner_ids = [
                 pos.positioner_id
@@ -730,7 +929,7 @@ class FPS(BaseFPS):
                 if pos.initialised and not pos.is_bootloader() and not pos.disabled
             ]
             if positioner_ids == []:
-                return True
+                return numpy.array([])
 
         command = await self.send_command(
             CommandID.GET_ACTUAL_POSITION,
@@ -751,7 +950,7 @@ class FPS(BaseFPS):
 
         await asyncio.gather(*update_position_commands)
 
-        return True
+        return self.get_positions()
 
     async def update_firmware_version(
         self,
@@ -770,6 +969,9 @@ class FPS(BaseFPS):
 
         """
 
+        if len(self.positioners) == 0:
+            return True
+
         if positioner_ids is None:
             positioner_ids = [0]
         elif not isinstance(positioner_ids, (list, tuple)):
@@ -804,27 +1006,31 @@ class FPS(BaseFPS):
 
         return True
 
-    async def stop_trajectory(self):
-        """Stops all the positioners.
+    async def stop_trajectory(self, clear_flags=False):
+        """Stops all the positioners without clearing collided flags.
 
         Parameters
         ----------
-        positioner_ids
-            The list of positioners to abort. If `None`, abort all positioners.
-        timeout
-            How long to wait before timing out the command. By default, just
-            sends the command and does not wait for replies.
+        clear_flags
+            If `True`, sends ``STOP_TRAJECTORY`` which clears collided
+            flags. Otherwise sends ``SEND_TRAJECTORY_ABORT``.
 
         """
 
-        await self.send_command(
-            "STOP_TRAJECTORY",
-            positioner_ids=0,
-            timeout=0,
-            now=True,
-        )
-
-        await self.send_command("TRAJECTORY_TRANSMISSION_ABORT", positioner_ids=None)
+        if clear_flags is False:
+            await self.send_command(
+                "SEND_TRAJECTORY_ABORT",
+                positioner_ids=None,
+                timeout=0,
+                now=True,
+            )
+        else:
+            await self.send_command(
+                "STOP_TRAJECTORY",
+                positioner_ids=0,
+                timeout=0,
+                now=True,
+            )
 
         # Check running command that are "move" and cancel them.
         assert isinstance(self.can, JaegerCAN)
@@ -842,7 +1048,7 @@ class FPS(BaseFPS):
         speed: Optional[Tuple[float, float]] = None,
         relative=False,
         force: bool = False,
-        use_sync_line=True,
+        use_sync_line: bool | None = None,
     ):
         """Sends a list of positioners to a given position.
 
@@ -916,10 +1122,9 @@ class FPS(BaseFPS):
         return await send_trajectory(self, *args, **kwargs)
 
     def abort(self):
-        """Aborts trajectories and stops positioners."""
+        """Aborts trajectories and stops positioners. Alias for `.stop_trajectory`."""
 
-        cmd = self.send_command(CommandID.STOP_TRAJECTORY, positioner_ids=0)
-        return asyncio.create_task(cmd)
+        return asyncio.create_task(self.stop_trajectory())
 
     async def send_to_all(self, *args, **kwargs):
         """Sends a command to all connected positioners.
@@ -964,6 +1169,139 @@ class FPS(BaseFPS):
 
         return status
 
+    async def save_snapshot(
+        self,
+        path: Optional[str | pathlib.Path] = None,
+        collision_buffer: float | None = None,
+        return_axes: bool = False,
+        highlight: int | None = None,
+    ) -> str | Axes:
+        """Creates a plot with the current arrangement of the FPS array.
+
+        Parameters
+        ----------
+        path
+            The path where to save the plot. Defaults to
+            ``/data/fps/snapshots/MJD/fps_snapshot_<SEQ>.pdf``.
+        collision_buffer
+            The collision buffer.
+        return_axes
+            If `True`, returns the matplotlib axes instead of saving the plot.
+
+        """
+
+        from jaeger.kaiju import get_snapshot
+
+        ax = await get_snapshot(collision_buffer=collision_buffer, highlight=highlight)
+
+        if return_axes is True:
+            return ax
+
+        if path is not None:
+            ax.figure.savefig(path)
+            return str(path)
+
+        mjd = int(Time.now().mjd)
+        dirpath = f"/data/fps/snapshots/{mjd}"
+        if not os.path.exists(dirpath):
+            os.makedirs(dirpath)
+
+        path_pattern = dirpath + "/fps_snapshot_*.pdf"
+        files = sorted(glob(path_pattern))
+
+        if len(files) == 0:
+            seq = 1
+        else:
+            seq = int(files[-1].split("_")[-1][0:4]) + 1
+
+        path = path_pattern.replace("*", f"{seq:04d}")
+        ax.figure.savefig(path)
+
+        return path
+
+    async def _handle_temperature(self):
+        """Handle positioners in low temperature."""
+
+        async def set_rpm(activate):
+            if activate:
+                rpm = config["low_temperature"]["rpm_cold"]
+                log.warning(f"Low temperature mode. Setting RPM={rpm}.")
+            else:
+                rpm = config["low_temperature"]["rpm_normal"]
+                log.warning(f"Disabling low temperature mode. Setting RPM={rpm}.")
+
+            config["positioner"]["motor_speed"] = rpm
+
+        async def set_idle_power(activate):
+            if activate:
+                ht = config["low_temperature"]["holding_torque_very_cold"]
+                log.warning("Very low temperature mode. Setting holding torque.")
+            else:
+                ht = config["low_temperature"]["holding_torque_normal"]
+                log.warning(
+                    "Disabling very low temperature mode. Setting holding torque."
+                )
+            await self.send_command(
+                CommandID.SET_HOLDING_CURRENT,
+                alpha=ht[0],
+                beta=ht[1],
+            )
+
+        sensor = config["low_temperature"]["sensor"]
+        cold = config["low_temperature"]["cold_threshold"]
+        very_cold = config["low_temperature"]["very_cold_threshold"]
+        interval = config["low_temperature"]["interval"]
+
+        while True:
+
+            try:
+                assert isinstance(self.ieb, IEB) and self.ieb.disabled is False
+                device = self.ieb.get_device(sensor)
+                temp = (await device.read())[0]
+
+                # Get the status without the temperature bits.
+                base_status = self.status & ~FPSStatus.TEMPERATURE_BITS
+
+                if temp <= very_cold:
+                    if self.status & FPSStatus.TEMPERATURE_NORMAL:
+                        await set_rpm(True)
+                        await set_idle_power(True)
+                    elif self.status & FPSStatus.TEMPERATURE_COLD:
+                        await set_idle_power(True)
+                    else:
+                        pass
+                    self.set_status(base_status | FPSStatus.TEMPERATURE_VERY_COLD)
+
+                elif temp <= cold:
+                    if self.status & FPSStatus.TEMPERATURE_NORMAL:
+                        await set_rpm(True)
+                    elif self.status & FPSStatus.TEMPERATURE_COLD:
+                        pass
+                    else:
+                        await set_idle_power(False)
+                    self.set_status(base_status | FPSStatus.TEMPERATURE_COLD)
+
+                else:
+                    if self.status & FPSStatus.TEMPERATURE_NORMAL:
+                        pass
+                    elif self.status & FPSStatus.TEMPERATURE_COLD:
+                        await set_rpm(False)
+                    else:
+                        await set_rpm(False)
+                        await set_idle_power(False)
+                    self.set_status(base_status | FPSStatus.TEMPERATURE_NORMAL)
+
+            except BaseException as err:
+                log.info(
+                    f"Cannot read device {sensor!r}. "
+                    f"Low-temperature mode will not be engaged: {err}",
+                )
+                base_status = self.status & ~FPSStatus.TEMPERATURE_BITS
+                self.set_status(base_status | FPSStatus.TEMPERATURE_UNKNOWN)
+                return
+
+            await asyncio.sleep(interval)
+
     async def shutdown(self):
         """Stops pollers and shuts down all remaining tasks."""
 
@@ -988,6 +1326,8 @@ class FPS(BaseFPS):
 
         self.loop.stop()
 
+        del self.__class__._instance[self.__class__]
+
     async def __aenter__(self):
         await self.initialise()
         return self
