@@ -191,8 +191,6 @@ class Positioner(StatusMixIn):
 
         self.alpha, self.beta = position
 
-        self._log(f"at position ({self.alpha:.2f}, {self.beta:.2f})")
-
     async def update_status(
         self,
         status: maskbits.PositionerStatus | int = None,
@@ -502,7 +500,7 @@ class Positioner(StatusMixIn):
         relative=False,
         force=False,
         use_trajectory=True,
-        use_sync_line=True,
+        use_sync_line=None,
     ) -> bool:
         """Moves positioner to a given position.
 
@@ -655,7 +653,7 @@ class Positioner(StatusMixIn):
                 await self.set_speed(*original_speed, force=force)
 
         if result is True:
-            self._log("Position reached.", logging.INFO)
+            self._log("Destination reached.", logging.INFO)
 
         return result
 
